-module(eAcs).

-export([
   match/2
]).

%% State is used to locate node, every node is a map
%% state 0 is the root node
%%
%% Goto:  State -> Map{Char -> State}
%% Ouput: State -> String
%% Failure: State -> State

%% try to find patterns in string
%% the match index starts from 1
%% @return [{StartIndex, EndIndex, Pattern},...]
match(String, {Goto, Failure, Output}) ->
   do_match(String, 0, {Goto, Failure, Output}, _Index = 1, _MatchList = []).


do_match([], _, _, _Index, MatchList) ->
   MatchList;
do_match([Char|Tail], State, {Goto, Failure, Output}, Index, MatchList) ->
   {NewState, NewMatchList} = do_match_inner(Char, State, {Goto, Failure, Output}, Index, MatchList),
   do_match(Tail, NewState, {Goto, Failure, Output}, Index + 1, NewMatchList).

%% {NewState, NewMatchList}
do_match_inner(Char, State, {Goto, Failure, Output}, Index, MatchList) ->
   #{State := Node} = Goto,
   case maps:find(Char, Node) of
      error ->
         case State =:= 0 of
            true ->
               {State, MatchList};
            false ->
               NextState = maps:get(State, Failure, 0),
               do_match_inner(Char, NextState, {Goto, Failure, Output}, Index, MatchList)
         end;
      {ok, NextState} ->
         NewMatchList = get_output(NextState, {Goto, Failure, Output}, Index, MatchList),
         {NextState, NewMatchList}
   end.

get_output(0, _, _Index, MatchList) ->
   MatchList;
get_output(State, {Goto, Failure, Output}, Index, MatchList) ->
   NewMatchList = case maps:find(State, Output) of
                     error -> MatchList;
                     {ok, Pattern} -> [{Index-length(Pattern) + 1, Index, Pattern} | MatchList]
                  end,
   FailureState = maps:get(State, Failure, 0),
   get_output(FailureState, {Goto, Failure, Output}, Index, NewMatchList).


